@@ -17,6 +17,7 @@
 #include <sys/syscall.h>
 #include <openssl/md5.h>
 #include <pwd.h>
+#define FUSE_USE_VERSION 30
 #include <fuse.h>
 #include <limits.h>
 #include <curl/curl.h>
@@ -237,7 +238,7 @@ void debug_print_flags(int flags)
 void debug_print_descriptor(struct fuse_file_info* info)
 {
   char file_path[MAX_PATH_SIZE];
-  openfile* of = (openfile *)info->fh;
+  openfile* of = (openfile *)(uintptr_t)info->fh;
   get_file_path_from_fd(of->fd, file_path, sizeof(file_path));
   debugf(DBG_LEVEL_EXT, KCYN "descriptor localfile=[%s] fd=%lld", file_path,
          of->fd);
